<?php

namespace SESP\Cache;

use ObjectCache;
use Language;
use BagOStuff;

/**
 * @ingroup SESP
 *
 * @licence GNU GPL v2+
 * @since 1.2.0
 *
 * @author mwjames
 */
class MessageCache {

	/** @var MessageCache[] */
	private static $instance = array();

	/** @var Language */
	protected $language = null;

	protected $touched = null;
	protected $cacheTimeOffset = null;
	protected $messages = null;
	protected $cache = null;

	/**
	 * @since 1.2.0
	 *
	 * @param Language|null $language
	 * @param integer|null $cacheTimeOffset
	 */
	public function __construct( Language $language = null, $cacheTimeOffset = null ) {
		$this->language = $language;
		$this->cacheTimeOffset = $cacheTimeOffset;
	}

	/**
	 * @since 1.4
	 *
	 * @param Language $language
	 */
	public function setLanguage( Language $language ) {
		$this->language = $language;
	}

	/**
	 * @since 1.4
	 *
	 * @return MessageCache
	 */
	public function inUserLanguage() {
		$this->language = $GLOBALS['wgLang'];
		return $this;
	}

	/**
	 * @since 1.2.0
	 */
	public static function clear() {
		self::$instance = array();
	}

	/**
	 * @since 1.2.0
	 *
	 * MessageCache::ByLanguage( Language::factory( 'en' ) )->purge()
	 *
	 * @return MessageCache
	 */
	public function purge() {
		$this->getCache()->delete( $this->getCacheId() );
		return $this;
	}

	/**
	 * @since 1.2.0
	 *
	 * @param integer $cacheTimeOffset
	 *
	 * @return MessageCache
	 */
	public function setCacheTimeOffset( $cacheTimeOffset ) {
		$this->cacheTimeOffset = $cacheTimeOffset;
		return $this;
	}

	/**
	 * @since 1.2.0
	 *
	 * @param BagOStuff $cache
	 */
	public function setCache( BagOStuff $cache ) {
		$this->cache = $cache;
		return $this;
	}

	/**
	 * @since 1.2.0
	 *
	 * @return string
	 */
	public function getCacheId() {
		return $this->getCachePrefix() . ':sesp:mcache:' . $this->language->getCode();
	}

	/**
	 * @since 1.2.0
	 *
	 * @param string $key
	 *
	 * @return string
	 */
	public function get( $key /* arguments */ ) {

		$arguments = func_get_args();

		if ( $this->messages === null ) {
			$this->messages = $this->fetchMessagesFromCache();
		}

		$key = implode( '#', $arguments );

		if ( isset( $this->messages[ $key ] ) ) {
			return $this->messages[ $key ];
		}

		return $this->getTextMessage( $key, $arguments );
	}

	protected function getTextMessage( $key, $arguments ) {

		$this->messages[ $key ] = wfMessage( $arguments )->inLanguage( $this->language )->text();
		$this->updateMessagesToCache();

		return $this->messages[ $key ];
	}

	protected function updateMessagesToCache() {

		$messagesToBeCached = array(
			'touched'  => $this->getTouched(),
			'messages' => $this->messages
		);

		return $this->getCache()->set( $this->getCacheId(), $messagesToBeCached );
	}

	protected function fetchMessagesFromCache() {

		$cached = $this->getCache()->get( $this->getCacheId() );

		if ( isset( $cached['touched'] ) && isset( $cached['messages'] ) && $cached['touched'] === $this->getTouched() ) {
			return $cached['messages'];
		}

		return null;
	}

	protected function getCache() {

		if ( !$this->cache instanceOf BagOStuff ) {
			$this->cache = ObjectCache::getInstance( $GLOBALS['sespCacheType'] );
		}

		return $this->cache;
	}

	protected function getCachePrefix() {
		return $GLOBALS['wgCachePrefix'] === false ? wfWikiID() : $GLOBALS['wgCachePrefix'];
	}

	protected function getTouched() {

		if ( $this->touched === null ) {
			$this->touched = $this->getMessageFileModificationTime() . $this->cacheTimeOffset;
		}

		return $this->touched;
	}

	protected function getMessageFileModificationTime() {

		if ( method_exists( $this->language, 'getJsonMessagesFileName' )  ) {
			$file = $this->language->getJsonMessagesFileName( $this->language->getCode() );
		} else {
			$file = $this->language->getMessagesFileName( $this->language->getCode() );
		}

		// #60
		if ( is_readable( $file ) ) {
			return filemtime( $file );
		}

		return microtime();
	}

}
